# -*- coding: utf-8 -*-
"""decorators -- decorators for owyl behavior trees.



Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 41 $\n
$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 41 $"[6:-2]
__date__ = "$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $"[7:-2]

import time

import core

__all__ = ['identity', 'repeatUntilFail', 'repeatUntilSucceed',
           'flip', 'repeatAlways', 'limit']

@core.parent_task
def identity(child, **kwargs):
    """Transparent decorator. Pass yielded values from child unchanged.
    """
    result = None
    child = child(**kwargs)
    while result is None:
        result = (yield child)
    yield result

@core.parent_task
def flip(child, **kwargs):
    """NOT decorator. Pass yielded values from child with the boolean flipped.

    Yielded values of "None" are passed unchanged.
    """
    result = None
    child = child(**kwargs)
    while result is None:
        result = (yield child)
    yield not result

@core.parent_task
def repeatAlways(child, **kwargs):
    """Perpetually iterate over the child, regardless of return value.
    """
    result = None
    while True:
        try:
            visitor = core.visit(child, **kwargs)
        except StopIteration:
            continue
        while result is None:
            try:
                result = (yield visitor.next())
            except StopIteration:
                yield None
                break
        

@core.parent_task
def repeatUntilFail(child, **kwargs):
    """Repeatedly iterate over the child until it fails.

    @keyword final_value: Value to return on failure.
    @type final_value: C{True} or C{False}
    """
    final_value = kwargs.pop('final_value', False)
    result = None
    child = child(**kwargs)
    while result is None:
        try:
            result = (yield child)
            if result is False:
                break
            else:
                yield None # Yield to other tasks.
                result = None
        except StopIteration:
            result = None
    yield final_value

@core.parent_task
def repeatUntilSucceed(child, **kwargs):
    """Repeatedly iterate over the child until it succeeds.

    @keyword final_value: Value to return on failure.
    @type final_value: C{True} or C{False}
    """
    final_value = kwargs.pop('final_value', True)
    result = None
    child = child(**kwargs)
    while result is None:
        try:
            result = (yield child)
            if result is True:
                break
            else:
                yield None # Yield to other tasks.
                result = None
        except StopIteration:
            result = None
    yield final_value

@core.parent_task
def limit(child, **kwargs):
    """Limit the child to only iterate once every period.

    Otherwise, act as an identity decorator.

    @keyword limit_period: how often to run the child, in seconds.
    """
    nowtime = time.time
    last_run = nowtime()
    period = kwargs.get('limit_period', 1.0)
    result = None
    visitor = core.visit(child, **kwargs)
    while True:
        now = nowtime()
        since_last = now - last_run
        if (since_last) <= period:
            yield None
            continue
        last_run = nowtime()
        result = visitor.next()
        yield result



# -*- coding: utf-8 -*-
"""blackboard -- basic blackboard behaviors for Owyl



Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 30 $\n
$Date: 2009-01-08 07:39:11 -0600 (Thu, 08 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 30 $"[6:-2]
__date__ = "$Date: 2009-01-08 07:39:11 -0600 (Thu, 08 Jan 2009) $"[7:-2]


from collections import defaultdict

import core

__all__ = ['checkBB', 'setBB',]

class Blackboard(defaultdict):
    """A dict that defaults values to None.
    """
    def __init__(self, **kwargs):
        default = lambda: None
        super(Blackboard, self).__init__(default, **kwargs)

@core.task
def checkBB(**kwargs):
    """Check a value on the blackboard.

    @keyword blackboard: The blackboard object.

    @keyword key: The name of a key on the blackboard.
    @type key: A hashable object

    @keyword check: A function that takes the value on the blackboard
                    and returns a boolean.
    """
    bb = kwargs['blackboard']
    key = kwargs['key']
    check = kwargs.get('check', lambda x: x is not None)
    value = bb[key]
    result = check(value) and True or False # Always return a boolean.
    yield result

@core.task
def setBB(**kwargs):
    """Set a value on the blackboard.

    @keyword blackboard: The blackboard object.

    @keyword key: The name of a key on the blackboard.
    @type key: A hashable object

    @keyword value: The value to set on the key.
    """
    bb = kwargs['blackboard']
    key = kwargs['key']
    value = kwargs['value']
    bb[key] = value
    yield True





"""remove -- remove command for vellum.
"""
import os
import shutil

def remove(scribe, paths=[], force=False):
    """
    Takes a dict with "paths" and "force" and then removes each of
    those paths, forcing removal if directories are not empty.  Paths
    can be files or directories. WARNING: If force is true,
    directories will be removed along with ALL OF THEIR CONTENTS.
    """
    assert isinstance(paths, list), "remove expects a list as the expression"
    for a_path in paths:
        a_path = os.path.expanduser(scribe.interpolate('remove', a_path))
        scribe.log("remove: %s" % a_path)
        if (os.path.exists(a_path) and not scribe.option("dry_run")):
            if os.path.isdir(a_path):
                try:
                    os.rmdir(a_path)
                except OSError:
                    if force:
                        scribe.log(" remove: forcing the removal of non-empty directory %s" % a_path)
                        shutil.rmtree(a_path)
                    else:
                        scribe.log(" remove: %s is a non-empty directory." % (a_path))
            else:
                try:
                    os.unlink(a_path)
                except Exception, e:
                    err_msg = "  %s: %s. Could not remove %s"
                    scribe.log(err_msg % (e.__class__.__name__, e.args, a_path))




# -*- coding: utf-8 -*-
"""owyl -- Owyl Behavior Trees

Behavior Trees are a form of U{hierarchical
logic<http://aigamedev.com/hierarchical-logic>}, and are quite useful
and flexible for implementing game AI.

Owyl implements a behavior tree using nested iterators/generators. A
top-level generator function, L{visit} (implementing the Visitor
Pattern), iterates through the tree, descending into child generators
as they are yielded, and passing yielded termination status values to
the parent generators.

Nested Generators
=================

  Tasks in the behavior tree are implemented as iterators (typically
  being generator functions wrapped by the L{task} decorator
  function).

  The first call to the task is at tree-building time. All passed
  arguments should be for static initialization. The task should
  return a factory function which itself should return an iterator.

  The factory function should accept **kwargs, which should be
  combined with the initialization keyword arugments and passed to the
  iterator at construction time.

  The iterator must yield values of None, True, False, or a child
  iterator. An iterator that yields child iterators must be ready to
  accept values yielded by the child. (See Termination Status Values,
  below.)


Termination Status Values
=========================

  As mentioned, iterators in the tree may yield values of None, True,
  False, or a child iterator:

    - B{None:} May be used to defer execution for another pass from the
      scheduler. An iterator yielding None will be queried again.

    - B{True:} Termination value signalling successful execution.

    - B{False:} Termination value signalling unsuccessful execution, or
      failure. Note: this is not considered an error value.

  True errors or exceptions should C{raise} the appropriate C{Error}
  or C{Exception}.
  
  For more information, see the discussion at
  U{http://aigamedev.com/hierarchical-logic/termination-status}.

For more information on Behavior Trees and hierarchical logic, please
see U{http://aigamedev.com/hierarchical-logic} and
U{http://aigamedev.com/hierarchical-logic/advice-2}.

Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 26 $\n
$Date: 2009-01-07 09:15:25 -0600 (Wed, 07 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 26 $"[6:-2]
__date__ = "$Date: 2009-01-07 09:15:25 -0600 (Wed, 07 Jan 2009) $"[7:-2]

from core import *
from decorators import *



# -*- coding: utf-8 -*-
"""stack -- stack implementation for owyl



Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 27 $\n
$Date: 2009-01-07 13:46:38 -0600 (Wed, 07 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 27 $"[6:-2]
__date__ = "$Date: 2009-01-07 13:46:38 -0600 (Wed, 07 Jan 2009) $"[7:-2]

__all__ = "EmptyError Stack".split()

EmptyError = IndexError

class Stack(list):
    """A list with a push method.
    """
    push = list.append



# -*- coding: utf-8 -*-
"""core -- core behaviors for Owyl.

Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 41 $\n
$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 41 $"[6:-2]
__date__ = "$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $"[7:-2]

import sys

try:
    from mx.Stack import Stack, EmptyError
except ImportError:
    from stack import Stack, EmptyError

RETURN_VALUES = (True, False, None)

__all__ = ['task', 'taskmethod', 'parent_task', 'parent_taskmethod', 'visit', 
           'succeed', 'fail', 'succeedAfter', 'failAfter',
           'sequence', 'selector', 'parallel', 'PARALLEL_SUCCESS',
           'throw', 'catch']

def task(func):
    """Task decorator.

    Decorate a generator function to produce a re-usable generator
    factory for the given task.
    """
    def initTask(**initkwargs):
        def makeIterator(**runkwargs):
            runkwargs.update(initkwargs)
            iterator = func(**runkwargs)
            return iterator
        makeIterator.__name__ = func.__name__
        makeIterator.__doc__ = func.__doc__
        return makeIterator
    initTask.__doc__ = func.__doc__
    initTask.__name__ = func.__name__
    return initTask

def taskmethod(func):
    """Task decorator.

    Decorate a generator function to produce a re-usable generator
    factory for the given task.
    """
    def initTask(self, **initkwargs):
        def makeIterator(**runkwargs):
            runkwargs.update(initkwargs)
            iterator = func(self, **runkwargs)
            return iterator
        makeIterator.__name__ = func.__name__
        makeIterator.__doc__ = func.__doc__
        return makeIterator
    initTask.__doc__ = func.__doc__
    initTask.__name__ = func.__name__
    return initTask

def parent_task(func):
    """Parent task decorator.

    A parent task is a task that accepts children.

    Decorate a generator function to produce a re-usable generator
    factory for the given task.
    """
    def initTask(*children, **initkwargs):
        def makeIterator(**runkwargs):
            runkwargs.update(initkwargs)
            iterator = func(*children, **runkwargs)
            return iterator
        makeIterator.__name__ = func.__name__
        makeIterator.__doc__ = func.__doc__
        return makeIterator
    initTask.__doc__ = func.__doc__
    initTask.__name__ = func.__name__
    return initTask

def parent_taskmethod(func):
    """Parent task decorator.

    A parent task is a task that accepts children.

    Decorate a generator function to produce a re-usable generator
    factory for the given task.
    """
    def initTask(self, *children, **initkwargs):
        def makeIterator(**runkwargs):
            runkwargs.update(initkwargs)
            iterator = func(self, *children, **runkwargs)
            return iterator
        makeIterator.__name__ = func.__name__
        makeIterator.__doc__ = func.__doc__
        return makeIterator
    initTask.__doc__ = func.__doc__
    initTask.__name__ = func.__name__
    return initTask

def visit(tree, **kwargs):
    """Iterate over a tree of nested iterators.

    Apply the U{Visitor
    Pattern<http://en.wikipedia.org/wiki/Visitor_pattern>} to a tree
    of nested iterators. Iterators should yield True, False, None, or
    a child iterator. Values of True or False are passed back to the
    parent iterator. A value of None is silently ignored, and the
    current iterator will be queried again on the next pass.

    The visitor will yield None until the tree raises StopIteration,
    upon which the visitor will yield the last value yielded by the
    tree, and terminate itself with StopIteration.

    The visitor is essentially a micro-scheduler for a Behavior Tree
    implemented as a tree of nested iterators. For more information,
    see the discussion at
    U{http://aigamedev.com/programming-tips/scheduler}.
    """
    s = Stack()
    return_values = RETURN_VALUES

    current = tree(**kwargs)
    send_value = None
    send_ok = False
    while True:
        try:
            if send_ok:
                child = current.send(send_value)
                send_value = None
                send_ok = False
            else:
                child = current.next()

            if child in return_values:
                send_value = child
                yield send_value
            else:
                # Descend into child node
                s.push(current)
                current = child
                
        except StopIteration:
            try:
                current = s.pop()
                send_ok = True
            except EmptyError:
                raise StopIteration
        except Exception, e:
            try:
                # Give the parent task a chance to handle the exception.
                current = s.pop()
                current.throw(*sys.exc_info())
            except EmptyError:
                # Give up if the exception has propagated all the way
                # up the tree:
                raise e
        
@task
def succeed(**kwargs):
    """Always succeed.
    """
    yield True

@task
def fail(**kwargs):
    """Always fail.
    """
    yield False

@task
def succeedAfter(**kwargs):
    """Succeed after a given number of iterations.

    Yields 'None' 'after' times.

    @keyword after: How many iterations to succeed after.
    @type after: int
    """
    after = kwargs.pop('after', 1)
    for x in xrange(after):
        yield None
    yield True

@task
def failAfter(**kwargs):
    """Fail after a given number of iterations.

    Yields 'None' 'after' times.

    @keyword after: How many iterations to fail after.
    @type after: int
    """
    after = kwargs.pop('after', 1)
    for x in xrange(after):
        yield None
    yield False


@parent_task
def sequence(*children, **kwargs):
    """Run tasks in sequence until one fails.

    The sequence will run each task in sequence until one fails,
    returning a failure. If all fail, returns a success.

    For more information, see the discussion at
    U{http://aigamedev.com/hierarchical-logic/sequence}.

    @param children: tasks to run in sequence as children.
    """
    final_value = True
    for child in children:
        result = yield child(**kwargs)
        if not result and result is not None:
            final_value = False
            break
        
    yield final_value

@parent_task
def selector(*children, **kwargs):
    """Run tasks in sequence until one succeeds.

    The selector will run each task in sequence until one succeeds,
    returning a success. If all fail, returns a failure.

    For more information, see the discussion at
    U{http://aigamedev.com/hierarchical-logic/selector}.

    @param children: child tasks to select from.
    """
    final_value = False
    for child in children:
        result = (yield child(**kwargs))
        if result:
            final_value = True
            break
        
    yield final_value


class Enum(object):
    """Enum/namespace class. Cannot be implemented. 

    Subclass and add class variables.
    """
    def __init__(self): 
        raise NotImplementedError, "_Enum class object. Do not instantiate."
    
class PARALLEL_SUCCESS(Enum):
    """Success policy enumerator for parallel behavior.

    C{REQUIRE_ALL}: All child tasks must succeed.
    C{REQUIRE_ONE}: Only one child task must succeed.
    """
    REQUIRE_ALL = "ALL"
    REQUIRE_ONE = "ONE"

@parent_task
def parallel(*children, **kwargs):
    """Run tasks in parallel until the success policy is fulfilled or broken.

    If the success policy is met, return a success. If the policy is
    broken, return a failure.

    For more information, see the discussion at
    U{aigamedev.com/hierarchical-logic/parallel}.

    @param children: tasks to run in parallel as children.

    @keyword policy: The success policy. All must succeed, 
                   or only one must succeed.
    @type policy: C{PARALLEL_SUCCESS.REQUIRE_ALL} or 
                  C{PARALLEL_SUCCESS.REQUIRE_ONE}.
    """
    return_values = (True, False)
    policy = kwargs.pop('policy', PARALLEL_SUCCESS.REQUIRE_ONE)
    all_must_succeed = (policy == PARALLEL_SUCCESS.REQUIRE_ALL)
    visits = [visit(arg, **kwargs) for arg in children]
    final_value = True
    while True:
        try:
            # Run one step on each child per iteration.
            for child in visits:
                result = child.next()
                if result in return_values:
                    if not result and all_must_succeed:
                        final_value = False
                        break
                    elif result and not all_must_succeed:
                        final_value = True
                        break
                    else:
                        final_value = result
            yield None
        except StopIteration:
            break
        except EmptyError:
            break
    yield final_value

@task
def throw(**kwargs):
    """Throw (raise) an exception.

    @keyword throws: An Exception to throw.
    @type throws: C{Exception}

    @keyword throws_message: Text to instantiate C{throws} with.
    @type throws_message: C{str}
    """
    throws = kwargs.pop('throws', Exception)
    throws_message = kwargs.pop('throws_message', '')
    class gen(object):
        def __iter__(self):
            return self
        def next(self):
            raise throws(throws_message)
    return gen()

@parent_task
def catch(child, **kwargs):
    """Catch a raised exception from child and run an alternate branch.

    Note: this will not catch exceptions raised in the branch.

    @keyword caught: An Exception to catch.
    @type caught: C{Exception}

    @keyword branch: An alternate tree to visit when caught.
    """
    caught = kwargs.pop('caught', Exception)
    branch = kwargs.pop('branch', fail())
    
    result = None
    child = child(**kwargs)
    branch = branch(**kwargs)
    try:
        while result is None:
            result = (yield child)
    except caught, e:
        while result is None:
            result = (yield branch)
    yield result



# -*- coding: utf-8 -*-
"""testowyl -- some tests for owyl.

Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 37 $\n
$Date: 2009-01-09 09:46:56 -0600 (Fri, 09 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 37 $"[6:-2]
__date__ = "$Date: 2009-01-09 09:46:56 -0600 (Fri, 09 Jan 2009) $"[7:-2]

import unittest

import owyl
from owyl import blackboard
from owyl import decorators

class OwylTests(unittest.TestCase):
    """Tests for Owyl.

    Note: tests should run the tree twice to make sure that the
    constructed tree is re-usable.
    """
    def testSucceed(self):
        """Can we succeed?
        """
        s = owyl.succeed()
        t = s()
        self.assertEqual(t.next(), True)
        self.assertRaises(StopIteration, t.next)

        t = s()
        self.assertEqual(t.next(), True)
        self.assertRaises(StopIteration, t.next)


    def testFail(self):
        """Can we fail?
        """
        s = owyl.fail()
        t = s()
        self.assertEqual(t.next(), False)
        self.assertRaises(StopIteration, t.next)

        t = s()
        self.assertEqual(t.next(), False)
        self.assertRaises(StopIteration, t.next)


    def testVisitSequenceSuccess(self):
        """Can we visit a successful sequence?
        """
        tree = owyl.sequence(owyl.succeed(),
                             owyl.succeed(),
                             owyl.succeed())

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True, True, True, True])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True, True, True, True])


    def testVisitSequenceFailure(self):
        """Can we visit a failing sequence?
        """
        tree = owyl.sequence(owyl.succeed(),
                             owyl.succeed(),
                             owyl.fail(),
                             owyl.succeed())

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True, True, False, False])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True, True, False, False])
        

    def testVisitSelectorSuccess(self):
        """Can we visit a successful selector?
        """
        tree = owyl.selector(owyl.fail(),
                             owyl.fail(),
                             owyl.succeed(),
                             owyl.fail())

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False, False, True, True])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False, False, True, True])


    def testVisitSelectorFailure(self):
        """Can we visit a failing selector?
        """
        tree = owyl.selector(owyl.fail(),
                             owyl.fail(),
                             owyl.fail())

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False, False, False, False])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False, False, False, False])


    def testParallel_AllSucceed_Success(self):
        """Can we visit a suceeding parallel (all succeed)?
        """
        tree = owyl.parallel(owyl.sequence(owyl.succeed(),
                                           owyl.succeed()),
                             owyl.sequence(owyl.succeed(),
                                           owyl.succeed()),
                             policy=owyl.PARALLEL_SUCCESS.REQUIRE_ALL)
        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True,])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True,])


    def testParallel_OneSucceeds_Success(self):
        """Can we visit a suceeding parallel (one succeeds)?
        """
        tree = owyl.parallel(owyl.sequence(owyl.succeed(),
                                           owyl.succeed()),
                             owyl.sequence(owyl.succeed(),
                                           owyl.fail()),
                             policy=owyl.PARALLEL_SUCCESS.REQUIRE_ONE)
        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True,])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [True,])


    def testParallel_AllSucceed_Failure(self):
        """Can we visit a failing parallel (all succeed)?
        """
        tree = owyl.parallel(owyl.sequence(owyl.succeed(),
                                           owyl.fail()),
                             owyl.sequence(owyl.succeed(),
                                           owyl.succeed()),
                             policy=owyl.PARALLEL_SUCCESS.REQUIRE_ALL)
        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False,])


    def testParallel_OneSucceeds_Failure(self):
        """Can we visit a failing parallel (one succeeds)?
        """
        tree = owyl.parallel(owyl.sequence(owyl.fail(),
                                           owyl.fail()),
                             owyl.sequence(owyl.fail(),
                                           owyl.fail()),
                             policy=owyl.PARALLEL_SUCCESS.REQUIRE_ONE)
        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False,])

        v = owyl.visit(tree)

        results =  [x for x in v if x is not None]
        self.assertEqual(results, [False,])


    def testThrow(self):
        """Can we throw an exception within the tree?
        """
        tree = owyl.sequence(owyl.succeed(),
                             owyl.succeed(),
                             owyl.throw(throws=ValueError,
                                        throws_message="AUGH!!"),
                             )
        v = owyl.visit(tree)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)
        self.assertRaises(ValueError, v.next)

        v = owyl.visit(tree)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)
        self.assertRaises(ValueError, v.next)


    def testCatch(self):
        """Can we catch an exception thrown within the tree?
        """
        tree = owyl.sequence(owyl.succeed(),
                             owyl.succeed(),
                             owyl.catch(owyl.throw(throws=ValueError,
                                                   throws_message="AUGH!!"),
                                        caught=ValueError, 
                                        branch=owyl.succeed())
                             )
        v = owyl.visit(tree)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)

        v = owyl.visit(tree)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)


    def testCatchIgnoresOthers(self):
        """Does catch ignore other exceptions thrown within the tree?
        """
        tree = owyl.sequence(owyl.succeed(),
                             owyl.succeed(),
                             owyl.catch(owyl.throw(throws=ValueError,
                                                   throws_message="AUGH!!"),
                                        caught=IndexError, 
                                        branch=owyl.succeed())
                             )
        v = owyl.visit(tree)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)
        self.assertRaises(ValueError, v.next)

        v = owyl.visit(tree)
        self.assertEqual(v.next(), True)
        self.assertEqual(v.next(), True)
        self.assertRaises(ValueError, v.next)


    def testIdentity(self):
        """Does identity pass on return values unchanged?
        """
        # Succeed after 5 iterations.
        after = 5
        tree = owyl.identity(owyl.succeedAfter(after=after))

        v = owyl.visit(tree)
        for x in xrange(after):
            self.assertEqual(v.next(), None)
        self.assertEqual(v.next(), True)

        v = owyl.visit(tree)
        for x in xrange(after):
            self.assertEqual(v.next(), None)
        self.assertEqual(v.next(), True)

        tree = owyl.identity(owyl.failAfter(after=after))

        v = owyl.visit(tree)
        for x in xrange(after):
            self.assertEqual(v.next(), None)
        self.assertEqual(v.next(), False)

        v = owyl.visit(tree)
        for x in xrange(after):
            self.assertEqual(v.next(), None)
        self.assertEqual(v.next(), False)


    def testCheckBB(self):
        """Can we check a value on a blackboard?
        """
        value = "foo"
        checker = lambda x: x == value

        bb = blackboard.Blackboard(value=value)
        tree = blackboard.checkBB(key='value',
                                  check=checker)
        
        # Note that we can pass in the blackboard at run-time.
        v = owyl.visit(tree, blackboard=bb)

        # Check should succeed.
        self.assertEqual(v.next(), True)

        v = owyl.visit(tree, blackboard=bb)
        self.assertEqual(v.next(), True)

        bb['value'] = 'bar'

        # Check should now fail.
        v = owyl.visit(tree, blackboard=bb)
        self.assertEqual(v.next(), False)

        v = owyl.visit(tree, blackboard=bb)
        self.assertEqual(v.next(), False)


    def testSetBB(self):
        """Can we set a value on a blackboard?
        """
        value = 'foo'
        checker = lambda x: x == value

        bb = blackboard.Blackboard(value='bar')
        tree = owyl.sequence(blackboard.setBB(key="value",
                                              value=value),
                             blackboard.checkBB(key='value',
                                                check=checker)
                             )
        
        # Note that we can pass in the blackboard at run-time.
        v = owyl.visit(tree, blackboard=bb)

        # Sequence will succeed if the check succeeds.
        result = [x for x in v][-1]
        self.assertEqual(result, True)

        v = owyl.visit(tree, blackboard=bb)
        result = [x for x in v][-1]
        self.assertEqual(result, True)


    def testRepeatUntilSucceed(self):
        """Can we repeat a behavior until it succeeds?
        """
        bb = blackboard.Blackboard() # 'value' defaults to None.
        checker = lambda x: x is not None

        parallel = owyl.parallel
        repeat = owyl.repeatUntilSucceed
        sequence = owyl.sequence
        checkBB = blackboard.checkBB
        setBB = blackboard.setBB

        tree = parallel(repeat(checkBB(key='value',
                                       check=checker),
                               final_value=True),

                        # That should fail until this sets the value:
                        owyl.selector(owyl.fail(),
                                      owyl.fail(),
                                      setBB(key='value',
                                            value='foo')),
                        policy=owyl.PARALLEL_SUCCESS.REQUIRE_ALL)

        v = owyl.visit(tree, blackboard=bb)
        results = [x for x in v]
        result = results[-1]
        self.assertEqual(result, True)

        # Need to reset the blackboard to get the same results.
        bb = blackboard.Blackboard() # 'value' defaults to None.
        v = owyl.visit(tree, blackboard=bb)
        results = [x for x in v]
        result = results[-1]
        self.assertEqual(result, True)


    def testRepeatUntilFail(self):
        """Can we repeat a behavior until it fails?
        """
        bb = blackboard.Blackboard(value="foo")
        checker = lambda x: x and True or False # must eval to True

        parallel = owyl.parallel
        repeat = owyl.repeatUntilFail
        sequence = owyl.sequence
        flip = decorators.flip # 'NOT' operator
        checkBB = blackboard.checkBB
        setBB = blackboard.setBB

        tree = parallel(repeat(checkBB(key='value',
                                            check=checker),
                                    final_value=True),

                        # That should succeed until this sets the value:
                        owyl.selector(owyl.fail(),
                                      owyl.fail(),
                                      setBB(key='value',
                                            value=None)),
                        policy=owyl.PARALLEL_SUCCESS.REQUIRE_ALL)

        v = owyl.visit(tree, blackboard=bb)
        results = [x for x in v]
        result = results[-1]
        self.assertEqual(result, True)

        # Need to reset the blackboard to get the same results.
        bb = blackboard.Blackboard(value="foo")
        v = owyl.visit(tree, blackboard=bb)
        results = [x for x in v]
        result = results[-1]
        self.assertEqual(result, True)


if __name__ == "__main__":
    runner = unittest
    try:
        import testoob
        runner = testoob
    except ImportError:
        pass
    runner.main()



# -*- coding: utf-8 -*-
"""setup -- setuptools setup file for Owyl.

$Author: david.eyk $\n
$Rev: 41 $\n
$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 41 $"
__date__ = "$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $"[7:-2]

__version__ = "0.3"
__release__ = '.r'.join((__version__, __revision__))

__description__ = "The goal of Owyl: provide a fast and flexible Behavior Tree library implemented in python."
__long_description__ = """You have Pyglet. You've got Rabbyt. But who do your sprites go to for advice? Owyl, of course.

The goal of Owyl: provide a fast and flexible Behavior Tree library implemented in python. For more information on Behavior Trees, see the articles at http://aigamedev.com/hierarchical-logic

"""
__classifiers__ = ["Development Status :: 3 - Alpha",
                   "Environment :: Console",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: BSD License",
                   "Natural Language :: English",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Topic :: Games/Entertainment",
                   "Topic :: Scientific/Engineering :: Artificial Intelligence",
                   "Topic :: Software Development :: Libraries",]

import sys

try:
    import setuptools
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()

from setuptools import setup, find_packages

INSTALL_REQUIRES=[]
ZIP_SAFE = True

setup(
    name = "owyl",
    version = __version__,
    author = "David Eyk",
    author_email = "eykd@eykd.net",
    url = "http://code.google.com/p/owyl/",
    description = __description__,
    long_description = __long_description__,
    download_url = "http://code.google.com/p/owyl/downloads/list",
    classifiers = __classifiers__,

    package_dir = {'': 'src',},
    packages = find_packages('src'),

    include_package_data = True,
    exclude_package_data = {'src':['*.c', '*.h',  '*.pyx', '*.pxd', '*.g']},
    #data_files=['src/data',],

    install_requires=INSTALL_REQUIRES,
    zip_safe = ZIP_SAFE,

    test_suite = "nose.collector",
    )




# -*- coding: utf-8 -*-
"""
"""

import unittest

from math import radians, pi
pi_2 = pi/2.0

from cocos.director import director

from owyl import blackboard

import boids

class BoidsTest(unittest.TestCase):
    def setUp(self):
        director.init()
        self.bb = blackboard.Blackboard()
        

    def testFacing(self):
        """Can we find the rotation to a set of coordinates?
        """
        b = boids.Boid(self.bb)
        b.position = (0, 0)
        b.rotation = 0

        pos = (0, 5)
        expect = radians(0)
        rr = b.getFacing(*pos)
        self.assertEqual(rr, expect, 
                         "%s: got %s, expected %s" % (pos, rr, expect))

        pos = (5, 0)
        expect = radians(90)
        rr = b.getFacing(*pos)
        self.assertEqual(rr, expect, 
                         "%s: got %s, expected %s" % (pos, rr, expect))

        pos = (-5, 0)
        expect = radians(-90)
        rr = b.getFacing(*pos)
        self.assertEqual(rr, expect, 
                         "%s: got %s, expected %s" % (pos, rr, expect))

        pos = (0, -5)
        expect = radians(180)
        rr = b.getFacing(*pos)
        self.assertEqual(rr, expect, 
                         "%s: got %s, expected %s" % (pos, rr, expect))
        
    def testFindRotationDelta(self):
        """Can we find the change of rotation to match a facing?
        """
        b = boids.Boid(self.bb)

        current_match_delta = ((0, 90, 90),
                               (0, -90, -90),
                               (0, 180, 180),
                               (0, 270, -90),
                               (90, 180, 90),
                               (90, 0, -90),
                               (270, 180, -90),
                               (270, 0, 90),
                               (90, 720, -90),
                               )

        for current, match, delta in current_match_delta:
            current = radians(current)
            match = radians(match)
            delta = radians(delta)
            dr = b.findRotationDelta(current, match)
            self.assertEqual(dr, delta, 
                             "c: %s; m: %s; e: %s; got: %s" % (current,
                                                               match,
                                                               delta,
                                                               dr))
        

if __name__ == "__main__":
    try:
        import testoob
        testoob.main()
    except ImportError:
        unittest.main()
        



# -*- coding: utf-8 -*-
"""boids -- Boids implementation using Owyl behavior trees.

Note: this demo requires Pyglet, Rabbyt, and cocos2d.

Copyright 2008 David Eyk. All rights reserved.

$Author: david.eyk $\n
$Rev: 41 $\n
$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $
"""

__author__ = "$Author: david.eyk $"[9:-2]
__revision__ = "$Rev: 41 $"[6:-2]
__date__ = "$Date: 2009-01-15 22:37:46 -0600 (Thu, 15 Jan 2009) $"[7:-2]

import os

import random
from math import radians, degrees, sin, cos, pi, atan2
pi_2 = pi*2.0
pi_1_2 = pi/2.0
pi_1_4 = pi/4.0
pi_3_4 = (pi*3)/4

from operator import attrgetter, itemgetter
getX = attrgetter('x')
getY = attrgetter('y')
getR = attrgetter('rotation')

import memojito

import pyglet
pyglet.resource.path = [os.path.dirname(os.path.abspath(__file__)),]
pyglet.resource.reindex()

from cocos.director import director
from cocos.scene import Scene
from cocos.sprite import Sprite
from cocos.actions import FadeIn
from cocos.tiles import ScrollableLayer, ScrollingManager

from owyl import blackboard
from owyl import taskmethod, visit, succeed, fail
from owyl import sequence, selector, parallel, PARALLEL_SUCCESS
from owyl.decorators import repeatUntilFail, limit, repeatAlways

from rabbyt.collisions import collide_single


class Boid(Sprite):
    _img = pyglet.resource.image('triangle.png')
    _img.anchor_x = _img.width / 2
    _img.anchor_y = _img.height / 2

    boids = []

    def __init__(self, blackboard):
        super(Boid, self).__init__(self._img)
        self.scale = 0.05
        self.schedule(self.update)
        self.bb = blackboard
        self.boids.append(self)
        self.opacity = 0
        self.do(FadeIn(2))
        self.speed = 200
        self.bounding_radius = 5
        self.bounding_radius_squared = 25
        self.neighborhood_radius = 1000
        self.personal_radius = 20

        self.tree = self.buildTree()

    def buildTree(self):
        """Build the behavior tree.

        Building the behavior tree is as simple as nesting the
        behavior constructor calls.
        """
        tree = parallel(limit(repeatAlways(self.clearMemoes(), debug=True), 
                              limit_period=1),
                        
                        ### Velocity and Acceleration
                        #############################
                        repeatAlways(sequence(self.hasCloseNeighbors(),
                                              self.accelerate(rate=.01),
                                              ),
                                     ),
                        self.move(),
                        self.matchSpeed(match_speed=200, rate=.01),

                        ### Steering
                        ############
                        self.seek(goal=(0, 0), rate=2),
                        self.steerToMatchHeading(rate=1),
                        self.steerForSeparation(rate=3),
                        self.steerForCohesion(rate=1),

                        policy=PARALLEL_SUCCESS.REQUIRE_ALL
                        )
        return visit(tree, blackboard=self.bb)

    def canSee(self, other):
        """Return True if I can see the other boid.
        """
        dx = self.x - other.x
        dy = self.y - other.y
        return abs(self.getFacing(dx, dy)) < pi_1_2

    @memojito.memoizedproperty
    def others(self):
        """Find other boids that I can see.
        """
        return [b for b in self.boids if b is not self and self.canSee(b)]

    @property
    def neighbors(self):
        """Find the other boids in my neighborhood.
        """
        hood = (self.x, self.y, self.neighborhood_radius) # neighborhood
        n = collide_single(hood, self.others)
        return n

    @property
    def closest_neighbors(self):
        """Find the average position of the closest neighbors.
        """
        hood = (self.x, self.y, self.personal_radius)
        n = collide_single(hood, self.others)
        return n
    
    def findAveragePosition(self, *boids):
        """Return the average position of the given boids.
        """
        if not boids:
            return (0, 0)
        num_n = float(len(boids)) or 1
        avg_x = sum((getX(n) for n in boids))/num_n
        avg_y = sum((getY(n) for n in boids))/num_n
        return avg_x, avg_y


    def findAverageHeading(self, *boids):
        """Return the average heading of the given boids.
        """
        if not boids:
            return 0.0
        return sum((getR(b) for b in boids))/len(boids)
    

    def findRotationDelta(self, this_heading, that_heading):
        """Find the change in rotation required to match a given heading.
        
        Headings are given in radians.
        """
        
        dr = (that_heading - this_heading)
        while dr < -pi:
            dr += pi_2
        while dr > pi:
            dr -= pi_2
        return dr

    def getFacing(self, tx, ty):
        """Find the facing rotation to local coordinates tx, ty.
        """
        return -(atan2(ty, tx) - pi_1_2)

    @taskmethod
    def move(self, **kwargs):
        """Move the actor forward perpetually.
        """
        bb = kwargs['blackboard']
        while True:
            dt = bb['dt']
            r = radians(getR(self)) # rotation
            s = dt * self.speed
            self.x += sin(r) * s
            self.y += cos(r) * s
            yield None


    @taskmethod
    def seek(self, **kwargs):
        """Perpetually seek a goal position.

        @keyword rate: steering rate
        @keyword blackboard: shared blackboard
        """
        bb = kwargs['blackboard']
        rate = kwargs['rate']
        gx, gy = kwargs.get('goal', (0, 0))
        while True:
            dt = bb['dt']
            dx = gx-self.x
            dy = gy-self.y
            seek_heading = self.getFacing(dx, dy)
            my_heading = radians(self.rotation)
            
            rsize = degrees(self.findRotationDelta(my_heading, seek_heading))

            rchange = rsize * rate * dt
            self.rotation += rchange
            yield None
            

    @taskmethod
    def steerToMatchHeading(self, **kwargs):
        """Perpetually steer to match actor's heading to neighbors.
        
        @keyword blackboard: shared blackboard
        @keyword rate: steering rate
        """
        bb = kwargs['blackboard']
        rate = kwargs['rate']
        while True:
            dt = bb['dt'] or 0.01
            n_heading = radians(self.findAverageHeading(*self.neighbors))
            if n_heading is None:
                yield None
                continue
            my_heading = radians(self.rotation)
            
            rsize = degrees(self.findRotationDelta(my_heading, n_heading))

            # Factor in our turning rate and elapsed time.
            rchange = rsize * rate * dt
            
            self.rotation += rchange
            yield None

    @taskmethod
    def hasCloseNeighbors(self, **kwargs):
        """Check to see if we have close neighbors.
        """
        yield bool(self.closest_neighbors)

    @taskmethod
    def accelerate(self, **kwargs):
        """accelerate

        @keyword rate: The rate of acceleration (+ or -)
        """
        bb = kwargs['blackboard']
        rate = kwargs['rate']
        dt = bb['dt']
        self.speed = max(self.speed + rate * dt, 0)
        yield True

    @taskmethod
    def matchSpeed(self, **kwargs):
        """Accelerate to match the given speed.

        @keyword blackboard: A shared blackboard.
        @keyword match_speed: The speed to match.
        @keyword rate: The rate of acceleration.
        """
        bb = kwargs['blackboard']
        ms = kwargs['match_speed']
        rate = kwargs['rate']
        while True:
            if self.speed == ms:
                yield None
            dt = bb['dt']
            dv_size = ms - self.speed
            dv = dv_size * rate * dt
            self.speed += dv
            yield None
                                     
    @taskmethod
    def steerForSeparation(self, **kwargs):
        """Steer to maintain distance between self and neighbors.
        """
        bb = kwargs['blackboard']
        rate = kwargs['rate']
        while True:
            cn_x, cn_y = self.findAveragePosition(*self.closest_neighbors)
            
            dt = bb['dt']
            dx = self.x-cn_x
            dy = self.y-cn_y
            
            heading_away_from_neighbors = self.getFacing(dx, dy)
            flee_heading = heading_away_from_neighbors
            my_heading = radians(self.rotation)
            
            rsize = degrees(self.findRotationDelta(my_heading, flee_heading))
            
            # Factor in our turning rate and elapsed time.
            rchange = rsize * rate * dt
            
            self.rotation += rchange
            yield None

    @taskmethod
    def steerForCohesion(self, **kwargs):
        """Steer toward the average position of neighbors.
        """
        bb = kwargs['blackboard']
        rate = kwargs['rate']
        while True:
            neighbors = self.neighbors
            np_x, np_y = self.findAveragePosition(*neighbors)
            dt = bb['dt']
            dx = np_x-self.x
            dy = np_y-self.y
            seek_heading = self.getFacing(dx, dy)
            my_heading = radians(self.rotation)

            # Find the rotation delta
            rsize = degrees(self.findRotationDelta(my_heading, seek_heading))
            
            # Factor in our turning rate and elapsed time.
            rchange = rsize * rate * dt

            self.rotation += rchange
            yield None

    @taskmethod
    def clearMemoes(self, **kwargs):
        """Clear memoizations.
        """
        self.clear()
        yield True

    @memojito.clearbefore
    def clear(self):
        """Clear memoizations.
        """
        pass

    def update(self, dt):
        """Update this Boid's behavior tree.

        This gets scheduled in the __init__.
        """
        self.bb['dt'] = dt
        self.tree.next()


class SpriteLayer(ScrollableLayer):
    is_event_handler = True
    
    def __init__(self):
        super(SpriteLayer, self).__init__()
        self.manager = ScrollingManager()
        self.manager.add(self)
        self.active = None
        self.blackboard = blackboard.Blackboard()


    def makeBoids(self, how_many):
        boids = []
        for x in xrange(how_many):
            boid = Boid(self.blackboard)
            boid.position = (random.randint(0, 200),
                             random.randint(0, 200))
            boid.rotation = random.randint(1, 360)
            self.add(boid)
            boids.append(boid)

        return boids

    def on_enter(self):
        super(SpriteLayer, self).on_enter()
        self.makeBoids(50)[0]

        self.manager.set_focus(0, 0)


if __name__ == "__main__":
    director.init(resizable=True, caption="Owyl Behavior Tree Demo: Boids",
                  width=1024, height=768)
    s = Scene(SpriteLayer())
    director.run(s)
